#pragma once

// RobotAPI
#include <RobotAPI/interface/armem/query.h>

#include <RobotAPI/libraries/armem/core/wm/ice_conversions.h>
#include <RobotAPI/libraries/armem/core/SuccessHeader.h>
#include <RobotAPI/libraries/armem/core/DataMode.h>
#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>


namespace armarx::armem::client::query
{
    // #include <RobotAPI/libraries/armem/client/query/Builder.h>
    class Builder;
}

namespace armarx::armem::client
{
    using QueryBuilder = query::Builder;


    /**
     * @brief An update of an entity for a specific point in time.
     */
    struct QueryInput
    {
        armem::query::data::MemoryQuerySeq memoryQueries;
        DataMode dataMode;

        void addQueryTargetToAll(const armem::query::data::QueryTarget t)
        {
            for (const auto& memoryQuery : memoryQueries)
            {
                if (std::find(memoryQuery->targets.begin(), memoryQuery->targets.end(), t) == memoryQuery->targets.end())
                {
                    memoryQuery->targets.push_back(t);
                }
                for (const auto& coreSegmentQuery : memoryQuery->coreSegmentQueries)
                {
                    if (std::find(coreSegmentQuery->targets.begin(), coreSegmentQuery->targets.end(), t) == coreSegmentQuery->targets.end())
                    {
                        coreSegmentQuery->targets.push_back(t);
                    }
                    for (const auto& providerSegmentQuery : coreSegmentQuery->providerSegmentQueries)
                    {
                        if (std::find(providerSegmentQuery->targets.begin(), providerSegmentQuery->targets.end(), t) == providerSegmentQuery->targets.end())
                        {
                            providerSegmentQuery->targets.push_back(t);
                        }
                        for (const auto& entityQuery : providerSegmentQuery->entityQueries)
                        {
                            if (std::find(entityQuery->targets.begin(), entityQuery->targets.end(), t) == entityQuery->targets.end())
                            {
                                entityQuery->targets.push_back(t);
                            }
                        }
                    }
                }
            }
        }

        void replaceQueryTarget(const armem::query::data::QueryTarget search, const armem::query::data::QueryTarget replace)
        {
            for (const auto& memoryQuery : memoryQueries)
            {
                if (auto it = std::find(memoryQuery->targets.begin(), memoryQuery->targets.end(), search); it != memoryQuery->targets.end())
                {
                    memoryQuery->targets.push_back(replace);
                }
                for (const auto& coreSegmentQuery : memoryQuery->coreSegmentQueries)
                {
                    if (auto it = std::find(coreSegmentQuery->targets.begin(), coreSegmentQuery->targets.end(), search); it != coreSegmentQuery->targets.end())
                    {
                        coreSegmentQuery->targets.push_back(replace);
                    }
                    for (const auto& providerSegmentQuery : coreSegmentQuery->providerSegmentQueries)
                    {
                        if (auto it = std::find(providerSegmentQuery->targets.begin(), providerSegmentQuery->targets.end(), search); it != providerSegmentQuery->targets.end())
                        {
                            providerSegmentQuery->targets.push_back(replace);
                        }
                        for (const auto& entityQuery : providerSegmentQuery->entityQueries)
                        {
                            if (auto it = std::find(entityQuery->targets.begin(), entityQuery->targets.end(), search); it != entityQuery->targets.end())
                            {
                                entityQuery->targets.push_back(replace);
                            }
                        }
                    }
                }
            }
        }

        static QueryInput fromIce(const armem::query::data::Input& ice);
        armem::query::data::Input toIce() const;
    };


    /**
     * @brief Result of an `EntityUpdate`.
     */
    struct QueryResult : public detail::SuccessHeader
    {
        wm::Memory memory;


        static QueryResult fromIce(const armem::query::data::Result& ice);
        armem::query::data::Result toIce() const;

        friend std::ostream& operator<<(std::ostream& os, const QueryResult& rhs);
    };


    void toIce(armem::query::data::Input& ice, const QueryInput& input);
    void fromIce(const armem::query::data::Input& ice, QueryInput& input);

    void toIce(armem::query::data::Result& ice, const QueryResult& result);
    void fromIce(const armem::query::data::Result& ice, QueryResult& result);

}
