#include "armarx.h"

#include <IceUtil/Time.h>

#include <ArmarXCore/core/time/ice_conversions.h>

#include <RobotAPI/libraries/aron/common/aron_conversions/core.h>
#include <RobotAPI/libraries/aron/common/aron_conversions/stl.h>


void IceUtil::fromAron(const long& dto, IceUtil::Time& bo)
{
    bo = IceUtil::Time::microSeconds(dto);
}

void IceUtil::toAron(long& dto, const IceUtil::Time& bo)
{
    dto = bo.toMicroSeconds();
}


void IceUtil::fromAron(const IceUtil::Time& dto, armarx::DateTime& bo)
{
    fromIce(dto, bo);
}

void IceUtil::toAron(IceUtil::Time& dto, const armarx::DateTime& bo)
{
    toIce(dto, bo);
}


/* PackagePath */

void armarx::fromAron(const arondto::PackagePath& dto, PackagePath& bo)
{
    bo = {dto.package, dto.path};
}

void armarx::toAron(arondto::PackagePath& dto, const PackagePath& bo)
{
    const data::PackagePath icedto = bo.serialize();
    dto.package                    = icedto.package;
    dto.path                       = icedto.path;
}


void armarx::fromAron(const arondto::ClockType& dto, ClockType& bo)
{
    switch (dto.value)
    {
        case arondto::ClockType::Realtime:
            bo = ClockType::Realtime;
            break;
        case arondto::ClockType::Monotonic:
            bo = ClockType::Monotonic;
            break;
        case arondto::ClockType::Virtual:
            bo = ClockType::Virtual;
            break;
        case arondto::ClockType::Unknown:
            bo = ClockType::Unknown;
            break;
    }
}

void armarx::toAron(arondto::ClockType& dto, const ClockType& bo)
{
    switch (bo)
    {
        case ClockType::Realtime:
            dto = arondto::ClockType::Realtime;
            break;
        case ClockType::Monotonic:
            dto = arondto::ClockType::Monotonic;
            break;
        case ClockType::Virtual:
            dto = arondto::ClockType::Virtual;
            break;
        case ClockType::Unknown:
            dto = arondto::ClockType::Unknown;
            break;
    }
}

void armarx::fromAron(const arondto::Duration& dto, Duration& bo)
{
    bo = Duration::MicroSeconds(dto.microSeconds);
}

void armarx::toAron(arondto::Duration& dto, const Duration& bo)
{
    dto.microSeconds = bo.toMicroSeconds();
}

void armarx::fromAron(const arondto::Frequency& dto, Frequency& bo)
{
    Duration cycleDuration;
    fromAron(dto.cycleDuration, cycleDuration);
    bo = Frequency(cycleDuration);
}

void armarx::toAron(arondto::Frequency& dto, const Frequency& bo)
{
    arondto::Duration cycleDuration;
    toAron(cycleDuration, bo.toCycleDuration());
    dto.cycleDuration = cycleDuration;
}

void armarx::fromAron(const arondto::DateTime& dto, DateTime& bo)
{
    Duration timeSinceEpoch;
    fromAron(dto.timeSinceEpoch, timeSinceEpoch);
    ClockType clockType;
    fromAron(dto.clockType, clockType);
    bo = DateTime(timeSinceEpoch, clockType, dto.hostname);
}

void armarx::toAron(arondto::DateTime& dto, const DateTime& bo)
{
    arondto::Duration timeSinceEpoch;
    toAron(timeSinceEpoch, bo.toDurationSinceEpoch());
    dto.timeSinceEpoch = timeSinceEpoch;
    arondto::ClockType clockType;
    toAron(clockType, bo.clockType());
    dto.clockType = clockType;
    dto.hostname = bo.hostname();
}
