#include "Skill.h"

namespace armarx
{
    namespace skills
    {
        Skill::Skill(const SkillDescription& desc):
            description(desc)
        {
            // replace constructor if you want to have a specific logging tag
            Logging::setTag("armarx::skills::" + description.skillName);
        }

        std::thread Skill::installTimeoutCondition()
        {
            // start timeout check
            return std::thread{ [&](){
                    if (description.timeoutMs <= 0) return;
                    long skillStartedAt = IceUtil::Time::now().toMilliSeconds();
                    while(running and !stopRequested())
                    {
                        auto now = IceUtil::Time::now().toMilliSeconds();
                        if ((now - skillStartedAt) >= description.timeoutMs) notifyTimeoutReached();
                        std::this_thread::sleep_for(std::chrono::milliseconds(100));
                    }
                }
            };
        }

        void Skill::notifyTimeoutReached()
        {
            timeoutReached = true;
        }

        void Skill::notifyStopped()
        {
            stopped = true;
        }

        void Skill::reset()
        {
            started = IceUtil::Time::milliSeconds(-1);
            exited = IceUtil::Time::milliSeconds(-1);

            running = false;
            stopped = false;
            timeoutReached = false;
        }

        void Skill::init(const aron::data::DictPtr& params)
        {
            (void) params;

            ARMARX_IMPORTANT << "Initializing Skill '" << description.skillName << "'";

            // always called before execute (should not take longer than 100ms)
            running = true;
            started = IceUtil::Time::now();
            timeoutCheck = installTimeoutCondition();
        }

        void Skill::exit(const aron::data::DictPtr& params)
        {
            (void) params;

            ARMARX_IMPORTANT << "Exiting Skill '" << description.skillName << "'";

            // always called after execute (should not take longer than 100ms)
            running = false;
            if (timeoutCheck.joinable())
            {
                timeoutCheck.join();
            }
            exited = IceUtil::Time::now();
        }

        Skill::Status Skill::execute(const aron::data::DictPtr& params, const CallbackT& callback)
        {
            (void) params;

            ARMARX_IMPORTANT << "Executing Skill '" << description.skillName << "'";
            return Status::Succeeded;
        }

        Skill::Status Skill::initExecuteExit(const aron::data::DictPtr& params, const CallbackT& callback)
        {
            this->reset();
            this->init(params);
            auto ret = this->execute(params);
            this->exit(params);
            return ret;
        }

        bool Skill::stopRequested() const
        {
            return (stopped || timeoutReached);
        }
    }
}
